@@ -11,9 +11,10 @@
 #    under the License.
 
 import os
-import uuid
 
+from oslo_utils import uuidutils
 import requests
+import requests.auth
 import six
 from tempest.lib import exceptions
 
@@ -90,12 +91,12 @@ class AodhClientTest(base.ClientTestBase):
             result = self.aodh('alarm', params=params)
             self.assertEqual("", result)
 
-        _test(str(uuid.uuid4()))
+        _test(uuidutils.generate_uuid())
         _test('normal-alarm-name')
 
     def test_event_scenario(self):
 
-        PROJECT_ID = str(uuid.uuid4())
+        PROJECT_ID = uuidutils.generate_uuid()
 
         # CREATE
         result = self.aodh(u'alarm',
@@ -146,7 +147,7 @@ class AodhClientTest(base.ClientTestBase):
                                   ALARM_ID))
 
         # LIST
-        result = self.aodh('alarm', params="list")
+        result = self.aodh('alarm', params="list --filter all_projects=true")
         self.assertIn(ALARM_ID,
                       [r['alarm_id'] for r in self.parser.listing(result)])
         output_colums = ['alarm_id', 'type', 'name', 'state', 'severity',
@@ -184,17 +185,17 @@ class AodhClientTest(base.ClientTestBase):
                          [r['alarm_id'] for r in self.parser.listing(result)])
 
     def test_threshold_scenario(self):
-
-        PROJECT_ID = str(uuid.uuid4())
+        PROJECT_ID = uuidutils.generate_uuid()
 
         # CREATE
         result = self.aodh(u'alarm',
-                           params=(u"create --type threshold --name alarm_th "
+                           params=(u"create --type threshold --name "
+                                   "test_threshold_scenario "
                                    "-m meter_name --threshold 5 "
                                    "--project-id %s" % PROJECT_ID))
         alarm = self.details_multiple(result)[0]
         ALARM_ID = alarm['alarm_id']
-        self.assertEqual('alarm_th', alarm['name'])
+        self.assertEqual('test_threshold_scenario', alarm['name'])
         self.assertEqual('meter_name', alarm['meter_name'])
         self.assertEqual('5.0', alarm['threshold'])
 
@@ -217,7 +218,8 @@ class AodhClientTest(base.ClientTestBase):
         # CREATE FAIL MISSING PARAM
         self.assertRaises(exceptions.CommandFailed,
                           self.aodh, u'alarm',
-                          params=(u"create --type threshold --name alarm_th "
+                          params=(u"create --type threshold --name "
+                                  "test_threshold_scenario "
                                   "--project-id %s" % PROJECT_ID))
 
         # UPDATE
@@ -235,28 +237,28 @@ class AodhClientTest(base.ClientTestBase):
         alarm_show = self.details_multiple(result)[0]
         self.assertEqual(ALARM_ID, alarm_show["alarm_id"])
         self.assertEqual(PROJECT_ID, alarm_show["project_id"])
-        self.assertEqual('alarm_th', alarm_show['name'])
+        self.assertEqual('test_threshold_scenario', alarm_show['name'])
         self.assertEqual('meter_name', alarm_show['meter_name'])
         self.assertEqual('10.0', alarm_show['threshold'])
 
         # GET BY NAME
         result = self.aodh(
-            'alarm', params="show --name alarm_th")
+            'alarm', params="show --name test_threshold_scenario")
         alarm_show = self.details_multiple(result)[0]
         self.assertEqual(ALARM_ID, alarm_show["alarm_id"])
         self.assertEqual(PROJECT_ID, alarm_show["project_id"])
-        self.assertEqual('alarm_th', alarm_show['name'])
+        self.assertEqual('test_threshold_scenario', alarm_show['name'])
         self.assertEqual('meter_name', alarm_show['meter_name'])
         self.assertEqual('10.0', alarm_show['threshold'])
 
         # GET BY NAME AND ID ERROR
         self.assertRaises(exceptions.CommandFailed,
                           self.aodh, u'alarm',
-                          params=(u"show %s --name alarm_th" %
+                          params=(u"show %s --name test_threshold_scenario" %
                                   ALARM_ID))
 
         # LIST
-        result = self.aodh('alarm', params="list")
+        result = self.aodh('alarm', params="list --filter all_projects=true")
         self.assertIn(ALARM_ID,
                       [r['alarm_id'] for r in self.parser.listing(result)])
         output_colums = ['alarm_id', 'type', 'name', 'state', 'severity',
@@ -264,44 +266,49 @@ class AodhClientTest(base.ClientTestBase):
         for alarm_list in self.parser.listing(result):
             self.assertEqual(sorted(output_colums), sorted(alarm_list.keys()))
             if alarm_list["alarm_id"] == ALARM_ID:
-                self.assertEqual('alarm_th', alarm_list['name'])
+                self.assertEqual('test_threshold_scenario', alarm_list['name'])
 
         # LIST WITH PAGINATION
         # list with limit
         result = self.aodh('alarm',
-                           params="list --limit 1")
+                           params="list --filter all_projects=true --limit 1 ")
         alarm_list = self.parser.listing(result)
         self.assertEqual(1, len(alarm_list))
         # list with sort with key=name dir=asc
-        result = self.aodh('alarm',
-                           params="list --sort name:asc")
+        result = self.aodh(
+            'alarm',
+            params="list --filter all_projects=true --sort name:asc")
         names = [r['name'] for r in self.parser.listing(result)]
         sorted_name = sorted(names)
         self.assertEqual(sorted_name, names)
         # list with sort with key=name dir=asc and key=alarm_id dir=asc
         result = self.aodh(u'alarm',
-                           params=(u"create --type threshold --name alarm_th "
+                           params=(u"create --type threshold --name "
+                                   "test_threshold_scenario "
                                    "-m meter_name --threshold 5 "
                                    "--project-id %s" % PROJECT_ID))
         created_alarm_id = self.details_multiple(result)[0]['alarm_id']
-        result = self.aodh('alarm',
-                           params="list --sort name:asc --sort alarm_id:asc")
+        result = self.aodh(
+            'alarm',
+            params="list --filter all_projects=true --sort name:asc --sort "
+                   "alarm_id:asc")
         alarm_list = self.parser.listing(result)
         ids_with_same_name = []
         names = []
         for alarm in alarm_list:
             names.append(['alarm_name'])
-            if alarm['name'] == 'alarm_th':
+            if alarm['name'] == 'test_threshold_scenario':
                 ids_with_same_name.append(alarm['alarm_id'])
         sorted_ids = sorted(ids_with_same_name)
         sorted_names = sorted(names)
         self.assertEqual(sorted_names, names)
         self.assertEqual(sorted_ids, ids_with_same_name)
         # list with sort with key=name dir=desc and with the marker equal to
-        # the alarm_id of the alarm_th we created for this test.
-        result = self.aodh('alarm',
-                           params="list --sort name:desc "
-                                  "--marker %s" % created_alarm_id)
+        # the alarm_id of the test_threshold_scenario we created for this test.
+        result = self.aodh(
+            'alarm',
+            params="list --filter all_projects=true --sort name:desc "
+                   "--marker %s" % created_alarm_id)
         self.assertIn('alarm_tc',
                       [r['name'] for r in self.parser.listing(result)])
         self.aodh('alarm', params="delete %s" % created_alarm_id)
@@ -311,7 +318,7 @@ class AodhClientTest(base.ClientTestBase):
                            params=("list --query project_id=%s" % PROJECT_ID))
         alarm_list = self.parser.listing(result)[0]
         self.assertEqual(ALARM_ID, alarm_list["alarm_id"])
-        self.assertEqual('alarm_th', alarm_list['name'])
+        self.assertEqual('test_threshold_scenario', alarm_list['name'])
 
         # DELETE
         result = self.aodh('alarm', params="delete %s" % ALARM_ID)
@@ -335,19 +342,25 @@ class AodhClientTest(base.ClientTestBase):
 
     def test_composite_scenario(self):
 
-        project_id = str(uuid.uuid4())
+        project_id = uuidutils.generate_uuid()
+        res_id = uuidutils.generate_uuid()
         # CREATE
-        result = self.aodh(u'alarm',
-                           params=(u'create --type composite --name calarm1 '
-                                   ' --composite-rule \'{"or":[{"threshold"'
-                                   ': 0.8,"meter_name": "cpu_util",'
-                                   '"type": "threshold"},{"and": ['
-                                   '{"threshold": 200, "meter_name": '
-                                   '"disk.iops", "type": "threshold"},'
-                                   '{"threshold": 1000,"meter_name":'
-                                   '"network.incoming.packets.rate",'
-                                   '"type": "threshold"}]}]}\' '
-                                   '--project-id %s' % project_id))
+        result = self.aodh(
+            u'alarm',
+            params=(u'create --type composite --name calarm1 --composite-rule '
+                    '\'{"or":[{"threshold": 0.8, "metric": "cpu_util", '
+                    '"type": "gnocchi_resources_threshold", "resource_type": '
+                    '"generic", "resource_id": "%s", '
+                    '"aggregation_method": "mean"},'
+                    '{"and": [{"threshold": 200, "metric": "disk.iops", '
+                    '"type": "gnocchi_resources_threshold", "resource_type": '
+                    '"generic", "resource_id": "%s", '
+                    '"aggregation_method": "mean"},'
+                    '{"threshold": 1000, "metric": "memory",'
+                    '"type": "gnocchi_resources_threshold", "resource_type": '
+                    '"generic", "resource_id": "%s", '
+                    '"aggregation_method": "mean"}]}]}\' --project-id %s' %
+                    (res_id, res_id, res_id, project_id)))
         alarm = self.details_multiple(result)[0]
         alarm_id = alarm['alarm_id']
         self.assertEqual('calarm1', alarm['name'])
@@ -390,7 +403,7 @@ class AodhClientTest(base.ClientTestBase):
                                   alarm_id))
 
         # LIST
-        result = self.aodh('alarm', params="list")
+        result = self.aodh('alarm', params="list --filter all_projects=true")
         self.assertIn(alarm_id,
                       [r['alarm_id'] for r in self.parser.listing(result)])
         output_colums = ['alarm_id', 'type', 'name', 'state', 'severity',
@@ -480,8 +493,8 @@ class AodhClientTest(base.ClientTestBase):
     def test_set_get_alarm_state(self):
         result = self.aodh(
             'alarm',
-            params=("create --type threshold --name alarm_state_test "
-                    "-m meter_name --threshold 5"))
+            params=('create --type event --name alarm_state_test '
+                    '--query "traits.project_id=789;traits.resource_id=012"'))
         alarm = self.details_multiple(result)[0]
         alarm_id = alarm['alarm_id']
         result = self.aodh(
@@ -498,23 +511,65 @@ class AodhClientTest(base.ClientTestBase):
         self.assertEqual('ok', state_get['state'])
         self.aodh('alarm', params='delete %s' % alarm_id)
 
+    def test_update_type_event_composite(self):
+
+        res_id = uuidutils.generate_uuid()
+        # CREATE
+        result = self.aodh(u'alarm',
+                           params=(u"create --type event --name ev_alarm123"))
+        alarm = self.details_multiple(result)[0]
+        ALARM_ID = alarm['alarm_id']
+        self.assertEqual('ev_alarm123', alarm['name'])
+        self.assertEqual('*', alarm['event_type'])
+
+        # UPDATE TYPE TO COMPOSITE
+        result = self.aodh(
+            'alarm',
+            params=('update %s --type composite --composite-rule '
+                    '\'{"or":[{"threshold": 0.8, "metric": "cpu_util", '
+                    '"type": "gnocchi_resources_threshold", "resource_type": '
+                    '"generic", "resource_id": "%s", '
+                    '"aggregation_method": "mean"},'
+                    '{"and": [{"threshold": 200, "metric": "disk.iops", '
+                    '"type": "gnocchi_resources_threshold", "resource_type": '
+                    '"generic", "resource_id": "%s", '
+                    '"aggregation_method": "mean"},'
+                    '{"threshold": 1000, "metric": "memory",'
+                    '"type": "gnocchi_resources_threshold", "resource_type": '
+                    '"generic", "resource_id": "%s", '
+                    '"aggregation_method": "mean"}]}]}\'' %
+                    (ALARM_ID, res_id, res_id, res_id)))
+        alarm_updated = self.details_multiple(result)[0]
+        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
+        self.assertEqual('composite', alarm_updated['type'])
+        self.assertIn('composite_rule', alarm_updated)
+
+        # UPDATE TYPE TO EVENT
+        result = self.aodh(
+            'alarm', params=("update %s --type event"
+                             % ALARM_ID))
+        alarm_updated = self.details_multiple(result)[0]
+        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
+        self.assertEqual('event', alarm_updated['type'])
+        self.assertEqual('*', alarm_updated['event_type'])
+
+        # DELETE
+        result = self.aodh('alarm', params="delete %s" % ALARM_ID)
+        self.assertEqual("", result)
+
 
 class AodhClientGnocchiRulesTest(base.ClientTestBase):
 
     def test_gnocchi_resources_threshold_scenario(self):
 
-        PROJECT_ID = str(uuid.uuid4())
-        RESOURCE_ID = str(uuid.uuid4())
+        PROJECT_ID = uuidutils.generate_uuid()
+        RESOURCE_ID = uuidutils.generate_uuid()
 
         req = requests.post(
-            os.environ.get("GNOCCHI_ENDPOINT") + "/v1/resource/instance",
+            os.environ.get("GNOCCHI_ENDPOINT") + "/v1/resource/generic",
+            auth=requests.auth.HTTPBasicAuth('admin', ''),
             json={
-                "display_name": "myvm",
-                "flavor_id": "2", "host": "blah",
                 "id": RESOURCE_ID,
-                "image_ref": "http://image",
-                "project_id": "BD3A1E52-1C62-44CB-BF04-660BD88CD74D",
-                "user_id": "BD3A1E52-1C62-44CB-BF04-660BD88CD74D",
             })
         self.assertEqual(201, req.status_code)
 
@@ -524,7 +579,7 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
                                    "--type gnocchi_resources_threshold "
                                    "--name alarm_gn1 --metric cpu_util "
                                    "--threshold 80 "
-                                   "--resource-id %s --resource-type instance "
+                                   "--resource-id %s --resource-type generic "
                                    "--aggregation-method last "
                                    "--project-id %s"
                                    % (RESOURCE_ID, PROJECT_ID)))
@@ -536,7 +591,24 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
         self.assertEqual('last', alarm['aggregation_method'])
         self.assertEqual(RESOURCE_ID,
                          alarm['resource_id'])
-        self.assertEqual('instance', alarm['resource_type'])
+        self.assertEqual('generic', alarm['resource_type'])
+
+        # CREATE WITH --TIME-CONSTRAINT
+        result = self.aodh(
+            u'alarm',
+            params=(u"create --type gnocchi_resources_threshold "
+                    "--name alarm_tc --metric cpu_util --threshold 80 "
+                    "--resource-id %s --resource-type generic "
+                    "--aggregation-method last --project-id %s "
+                    "--time-constraint "
+                    "name=cons1;start='0 11 * * *';duration=300 "
+                    "--time-constraint "
+                    "name=cons2;start='0 23 * * *';duration=600 "
+                    % (RESOURCE_ID, PROJECT_ID)))
+        alarm = self.details_multiple(result)[0]
+        self.assertEqual('alarm_tc', alarm['name'])
+        self.assertEqual('80.0', alarm['threshold'])
+        self.assertIsNotNone(alarm['time_constraints'])
 
         # CREATE FAIL MISSING PARAM
         self.assertRaises(exceptions.CommandFailed,
@@ -544,7 +616,7 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
                           params=(u"create "
                                   "--type gnocchi_resources_threshold "
                                   "--name alarm1 --metric cpu_util "
-                                  "--resource-id %s --resource-type instance "
+                                  "--resource-id %s --resource-type generic "
                                   "--aggregation-method last "
                                   "--project-id %s"
                                   % (RESOURCE_ID, PROJECT_ID)))
@@ -569,11 +641,12 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
         self.assertEqual('90.0', alarm_show['threshold'])
         self.assertEqual('critical', alarm_show['severity'])
         self.assertEqual('last', alarm_show['aggregation_method'])
-        self.assertEqual('instance', alarm_show['resource_type'])
+        self.assertEqual('generic', alarm_show['resource_type'])
 
         # GET BY NAME
         result = self.aodh(
             'alarm', params="show --name alarm_gn1")
+        alarm_show = self.details_multiple(result)[0]
         self.assertEqual(ALARM_ID, alarm_show["alarm_id"])
         self.assertEqual(PROJECT_ID, alarm_show["project_id"])
         self.assertEqual('alarm_gn1', alarm_show['name'])
@@ -581,7 +654,7 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
         self.assertEqual('90.0', alarm_show['threshold'])
         self.assertEqual('critical', alarm_show['severity'])
         self.assertEqual('last', alarm_show['aggregation_method'])
-        self.assertEqual('instance', alarm_show['resource_type'])
+        self.assertEqual('generic', alarm_show['resource_type'])
 
         # GET BY NAME AND ID ERROR
         self.assertRaises(exceptions.CommandFailed,
@@ -590,7 +663,7 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
                                   ALARM_ID))
 
         # LIST
-        result = self.aodh('alarm', params="list")
+        result = self.aodh('alarm', params="list --filter all_projects=true")
         self.assertIn(ALARM_ID,
                       [r['alarm_id'] for r in self.parser.listing(result)])
         output_colums = ['alarm_id', 'type', 'name', 'state', 'severity',
@@ -600,6 +673,53 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
             if alarm_list["alarm_id"] == ALARM_ID:
                 self.assertEqual('alarm_gn1', alarm_list['name'])
 
+        # LIST WITH PAGINATION
+        # list with limit
+        result = self.aodh('alarm',
+                           params="list --filter all_projects=true --limit 1")
+        alarm_list = self.parser.listing(result)
+        self.assertEqual(1, len(alarm_list))
+        # list with sort with key=name dir=asc
+        result = self.aodh(
+            'alarm',
+            params="list --filter all_projects=true --sort name:asc")
+        names = [r['name'] for r in self.parser.listing(result)]
+        sorted_name = sorted(names)
+        self.assertEqual(sorted_name, names)
+        # list with sort with key=name dir=asc and key=alarm_id dir=asc
+        result = self.aodh(
+            u'alarm',
+            params=(u"create --type gnocchi_resources_threshold "
+                    "--name alarm_th --metric cpu_util --threshold 80 "
+                    "--resource-id %s --resource-type generic "
+                    "--aggregation-method last --project-id %s "
+                    % (RESOURCE_ID, PROJECT_ID)))
+        created_alarm_id = self.details_multiple(result)[0]['alarm_id']
+        result = self.aodh(
+            'alarm',
+            params="list --filter all_projects=true --sort name:asc "
+                   "--sort alarm_id:asc")
+        alarm_list = self.parser.listing(result)
+        ids_with_same_name = []
+        names = []
+        for alarm in alarm_list:
+            names.append(['alarm_name'])
+            if alarm['name'] == 'alarm_th':
+                ids_with_same_name.append(alarm['alarm_id'])
+        sorted_ids = sorted(ids_with_same_name)
+        sorted_names = sorted(names)
+        self.assertEqual(sorted_names, names)
+        self.assertEqual(sorted_ids, ids_with_same_name)
+        # list with sort with key=name dir=desc and with the marker equal to
+        # the alarm_id of the alarm_th we created for this test.
+        result = self.aodh(
+            'alarm',
+            params="list --filter all_projects=true --sort name:desc "
+                   "--marker %s" % created_alarm_id)
+        self.assertIn('alarm_tc',
+                      [r['name'] for r in self.parser.listing(result)])
+        self.aodh('alarm', params="delete %s" % created_alarm_id)
+
         # LIST WITH QUERY
         result = self.aodh('alarm',
                            params=("list --query project_id=%s" % PROJECT_ID))
@@ -629,7 +749,7 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
 
     def test_gnocchi_aggr_by_resources_scenario(self):
 
-        PROJECT_ID = str(uuid.uuid4())
+        PROJECT_ID = uuidutils.generate_uuid()
 
         # CREATE
         result = self.aodh(
@@ -639,8 +759,8 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
                     "gnocchi_aggregation_by_resources_threshold "
                     "--name alarm1 --metric cpu --threshold 80 "
                     "--query "
-                    '\'{"=": {"server_group": "my_group"}}\' '
-                    "--resource-type instance "
+                    '\'{"=": {"creator": "cr3at0r"}}\' '
+                    "--resource-type generic "
                     "--aggregation-method last "
                     "--project-id %s" % PROJECT_ID))
         alarm = self.details_multiple(result)[0]
@@ -649,8 +769,8 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
         self.assertEqual('cpu', alarm['metric'])
         self.assertEqual('80.0', alarm['threshold'])
         self.assertEqual('last', alarm['aggregation_method'])
-        self.assertEqual('instance', alarm['resource_type'])
-        self.assertEqual('{"=": {"server_group": "my_group"}}',
+        self.assertEqual('generic', alarm['resource_type'])
+        self.assertEqual('{"=": {"creator": "cr3at0r"}}',
                          alarm['query'])
 
         # CREATE FAIL MISSING PARAM
@@ -662,8 +782,8 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
                     "gnocchi_aggregation_by_resources_threshold "
                     "--name alarm1 --metric cpu "
                     "--query "
-                    '\'{"=": {"server_group": "my_group"}}\' '
-                    "--resource-type instance "
+                    '\'{"=": {"creator": "cr3at0r"}}\' '
+                    "--resource-type generic "
                     "--aggregation-method last "
                     "--project-id %s" % PROJECT_ID))
 
@@ -687,10 +807,10 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
         self.assertEqual('90.0', alarm_show['threshold'])
         self.assertEqual('critical', alarm_show['severity'])
         self.assertEqual('last', alarm_show['aggregation_method'])
-        self.assertEqual('instance', alarm_show['resource_type'])
+        self.assertEqual('generic', alarm_show['resource_type'])
 
         # LIST
-        result = self.aodh('alarm', params="list")
+        result = self.aodh('alarm', params="list --filter all_projects=true")
         self.assertIn(ALARM_ID,
                       [r['alarm_id'] for r in self.parser.listing(result)])
         output_colums = ['alarm_id', 'type', 'name', 'state', 'severity',
@@ -722,7 +842,7 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
 
     def test_gnocchi_aggr_by_metrics_scenario(self):
 
-        PROJECT_ID = str(uuid.uuid4())
+        PROJECT_ID = uuidutils.generate_uuid()
         METRIC1 = 'cpu'
         METRIC2 = 'cpu_util'
 
@@ -733,7 +853,7 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
                     "--type gnocchi_aggregation_by_metrics_threshold "
                     "--name alarm1 "
                     "--metrics %s "
-                    "--metrics %s "
+                    "--metric %s "
                     "--threshold 80 "
                     "--aggregation-method last "
                     "--project-id %s"
@@ -781,7 +901,7 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
         self.assertEqual('last', alarm_show['aggregation_method'])
 
         # LIST
-        result = self.aodh('alarm', params="list")
+        result = self.aodh('alarm', params="list --filter all_projects=true")
         self.assertIn(ALARM_ID,
                       [r['alarm_id'] for r in self.parser.listing(result)])
         for alarm_list in self.parser.listing(result):
@@ -819,3 +939,135 @@ class AodhClientGnocchiRulesTest(base.ClientTestBase):
             self.assertEqual(sorted(output_colums), sorted(alarm_list.keys()))
         self.assertNotIn(ALARM_ID,
                          [r['alarm_id'] for r in self.parser.listing(result)])
+
+    def test_update_gnresthr_gnaggrresthr(self):
+
+        RESOURCE_ID = uuidutils.generate_uuid()
+        # CREATE
+        result = self.aodh(u'alarm',
+                           params=(u"create "
+                                   "--type gnocchi_resources_threshold "
+                                   "--name alarm_gn123 --metric cpu_util "
+                                   "--resource-id %s --threshold 80 "
+                                   "--resource-type generic "
+                                   "--aggregation-method last "
+                                   % RESOURCE_ID))
+        alarm = self.details_multiple(result)[0]
+        ALARM_ID = alarm['alarm_id']
+        self.assertEqual('alarm_gn123', alarm['name'])
+        self.assertEqual('cpu_util', alarm['metric'])
+        self.assertEqual('80.0', alarm['threshold'])
+        self.assertEqual('last', alarm['aggregation_method'])
+        self.assertEqual('generic', alarm['resource_type'])
+
+        # UPDATE TYPE TO GNOCCHI_AGGREGATION_BY_RESOURCES_THRESHOLD
+        result = self.aodh(
+            'alarm', params=("update %s --type "
+                             "gnocchi_aggregation_by_resources_threshold "
+                             "--metric cpu --threshold 90 "
+                             "--query "
+                             '\'{"=": {"creator": "cr3at0r"}}\' '
+                             "--resource-type generic "
+                             "--aggregation-method last "
+                             % ALARM_ID))
+        alarm_updated = self.details_multiple(result)[0]
+        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
+        self.assertEqual('cpu', alarm_updated['metric'])
+        self.assertEqual('90.0', alarm_updated['threshold'])
+        self.assertEqual('last', alarm_updated['aggregation_method'])
+        self.assertEqual('generic', alarm_updated['resource_type'])
+        self.assertEqual('{"=": {"creator": "cr3at0r"}}',
+                         alarm_updated['query'])
+        self.assertEqual('gnocchi_aggregation_by_resources_threshold',
+                         alarm_updated['type'])
+
+        # UPDATE TYPE TO GNOCCHI_RESOURCES_THRESHOLD
+        result = self.aodh(
+            'alarm', params=("update %s "
+                             "--type gnocchi_resources_threshold "
+                             "--metric cpu_util "
+                             "--resource-id %s --threshold 80 "
+                             "--resource-type generic "
+                             "--aggregation-method last "
+                             % (ALARM_ID, RESOURCE_ID)))
+        alarm_updated = self.details_multiple(result)[0]
+        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
+        self.assertEqual('cpu_util', alarm_updated['metric'])
+        self.assertEqual('80.0', alarm_updated['threshold'])
+        self.assertEqual('last', alarm_updated['aggregation_method'])
+        self.assertEqual('generic', alarm_updated['resource_type'])
+        self.assertEqual('gnocchi_resources_threshold',
+                         alarm_updated['type'])
+
+        # DELETE
+        result = self.aodh('alarm', params="delete %s" % ALARM_ID)
+        self.assertEqual("", result)
+
+    def test_update_gnaggrresthr_gnaggrmetricthr(self):
+
+        METRIC1 = 'cpu'
+        METRIC2 = 'cpu_util'
+
+        # CREATE
+        result = self.aodh(
+            u'alarm',
+            params=(u"create "
+                    "--type "
+                    "gnocchi_aggregation_by_resources_threshold "
+                    "--name alarm123 --metric cpu --threshold 80 "
+                    "--query "
+                    '\'{"=": {"creator": "cr3at0r"}}\' '
+                    "--resource-type generic "
+                    "--aggregation-method last "))
+        alarm = self.details_multiple(result)[0]
+        ALARM_ID = alarm['alarm_id']
+        self.assertEqual('alarm123', alarm['name'])
+        self.assertEqual('cpu', alarm['metric'])
+        self.assertEqual('80.0', alarm['threshold'])
+        self.assertEqual('last', alarm['aggregation_method'])
+        self.assertEqual('generic', alarm['resource_type'])
+        self.assertEqual('{"=": {"creator": "cr3at0r"}}',
+                         alarm['query'])
+
+        # UPDATE TYPE TO GNOCCHI_AGGREGATION_BY_METRICS_THRESHOLD
+        result = self.aodh(
+            'alarm', params=("update %s --type "
+                             "gnocchi_aggregation_by_metrics_threshold "
+                             "--metrics %s "
+                             "--metrics %s "
+                             "--threshold 80 "
+                             "--aggregation-method last"
+                             % (ALARM_ID, METRIC1, METRIC2)))
+        alarm_updated = self.details_multiple(result)[0]
+        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
+        metrics = "[u'cpu', u'cpu_util']" if six.PY2 else "['cpu', 'cpu_util']"
+        self.assertEqual(metrics, alarm_updated['metrics'])
+        self.assertEqual('80.0', alarm_updated['threshold'])
+        self.assertEqual('last', alarm_updated['aggregation_method'])
+        self.assertEqual('gnocchi_aggregation_by_metrics_threshold',
+                         alarm_updated['type'])
+
+        # UPDATE TYPE TO GNOCCHI_AGGREGATION_BY_RESOURCES_THRESHOLD
+        result = self.aodh(
+            'alarm', params=("update %s --type "
+                             "gnocchi_aggregation_by_resources_threshold "
+                             "--metric cpu --threshold 80 "
+                             "--query "
+                             '\'{"=": {"creator": "cr3at0r"}}\' '
+                             "--resource-type generic "
+                             "--aggregation-method last "
+                             % ALARM_ID))
+        alarm_updated = self.details_multiple(result)[0]
+        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
+        self.assertEqual('cpu', alarm_updated['metric'])
+        self.assertEqual('80.0', alarm_updated['threshold'])
+        self.assertEqual('last', alarm_updated['aggregation_method'])
+        self.assertEqual('generic', alarm_updated['resource_type'])
+        self.assertEqual('{"=": {"creator": "cr3at0r"}}',
+                         alarm_updated['query'])
+        self.assertEqual('gnocchi_aggregation_by_resources_threshold',
+                         alarm_updated['type'])
+
+        # DELETE
+        result = self.aodh('alarm', params="delete %s" % ALARM_ID)
+        self.assertEqual("", result)
